package ee.yoursit.core.command.bukkit.api;

import dev.inventex.octa.data.convertible.Convertible;
import lombok.AllArgsConstructor;
import lombok.Getter;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.potion.PotionEffectType;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

@AllArgsConstructor
public class CommandContext {
    @NotNull
    @Getter
    private final CommandSender sender;

    @NotNull
    private final String[] args;

    public boolean isPlayer() {
        return sender instanceof Player;
    }

    @NotNull
    public Player getPlayer() {
        if (!isPlayer())
            throw new IllegalStateException("CommandSender is not a player");
        return (Player) sender;
    }

    public boolean isEmpty() {
        return args.length == 0;
    }

    public int size() {
        return args.length;
    }

    @NotNull
    public String[] getData() {
        return args;
    }

    @Nullable
    public String getStringRaw(int index) {
        return index >= 0 && index < args.length
            ? args[index]
            : null;
    }

    @NotNull
    public Convertible<String, String> getString(int index) {
        return index >= 0 && index < args.length
            ? Convertible.completed(args[index])
            : Convertible.empty();
    }

    @NotNull
    public Convertible<String, Integer> getInt(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], Integer::parseInt)
            : Convertible.empty();
    }

    @NotNull
    public Convertible<String, Float> getFloat(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], Float::parseFloat)
            : Convertible.empty();
    }

    @NotNull
    public Convertible<String, Double> getDouble(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], Double::parseDouble)
            : Convertible.empty();
    }

    @NotNull
    public Convertible<String, Boolean> getBoolean(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], value -> switch (value.toLowerCase()) {
                case "enable", "allow", "true", "yes", "on", "y" -> true;
                default -> false;
            })
            : Convertible.empty();
    }

    @NotNull
    public Convertible<String, Player> getPlayer(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], Bukkit::getPlayer)
            : Convertible.empty();
    }

    @NotNull
    public Convertible<String, Player> getPlayerExact(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], Bukkit::getPlayerExact)
            : Convertible.empty();
    }

    @NotNull
    public Convertible<String, World> getWorld(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], Bukkit::getWorld)
            : Convertible.empty();
    }

    public Convertible<String, Material> getMaterial(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], s -> Material.getMaterial(s.toUpperCase()))
            : Convertible.empty();
    }

    public Convertible<String, PotionEffectType> getPotionEffectByName(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], s -> PotionEffectType.getByName(s.toUpperCase()))
            : Convertible.empty();
    }

    public Convertible<String, PotionEffectType> getPotionEffectById(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], s -> PotionEffectType.getById(Integer.parseInt(s)))
            : Convertible.empty();
    }

    public Convertible<String, PotionEffectType> getPotionEffect(int index) {
        return index >= 0 && index < args.length
            ? Convertible.of(args[index], s -> {
                PotionEffectType type = PotionEffectType.getByName(s.toUpperCase());
                if (type != null)
                    return type;

                return PotionEffectType.getById(Integer.parseInt(s));
            })
            : Convertible.empty();
    }

    public void reply(String message) {
        sender.sendMessage(message);
    }
}
