package ee.yoursit.core.command.bukkit.impl.invite;

import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.command.bukkit.api.Command;
import ee.yoursit.core.command.bukkit.api.CommandContext;
import ee.yoursit.core.command.bukkit.api.CommandInfo;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.data.PlayerData;
import ee.yoursit.core.database.entity.api.EntitySerializer;
import ee.yoursit.core.database.entity.impl.User;
import ee.yoursit.core.database.entity.impl.Verification;
import ee.yoursit.core.util.Constants;
import lombok.SneakyThrows;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

@CommandInfo(name = "verify")
public class VerifyCommand extends Command {
    @Override
    public void onCommand(@NotNull CommandContext context) throws Exception {
        Player player = context.getPlayer();
        PlayerData data = YsmpCore.INSTANCE
            .getDataManager()
            .get(player);

        if (!data.isVerifying()) {
            context.reply(Messages.INVITE_ALREADY_VERIFIED);
            return;
        }

        String code = context.getString(0).get();

        YsmpCore.INSTANCE
            .getVerificationService()
            .getVerification(code)
            .then(verification -> verifyUser(player, data, verification))
            .except(e -> data.kick(Messages.INVITE_INVALID_CODE));
    }

    @SneakyThrows
    private void verifyUser(Player player, PlayerData data, Verification verification) {
        JDA jda = YsmpCore.INSTANCE
            .getClient()
            .getJda();

        Guild guild = jda.getGuildById(Constants.YSMP_SERVER);
        assert guild != null;

        Member member = guild.getMemberById(verification.getDiscordId().get());
        if (member == null) {
            player.sendMessage(Messages.INVITE_NOT_JOINED_GUILD);
            return;
        }

        User user = createUser(player, verification);

        YsmpCore.INSTANCE
            .getUserService()
            .addUser(user)
            .get();

        YsmpCore.INSTANCE
            .getVerificationService()
            .removeVerification(verification)
            .get();

        data.setVerifying(false);

        Role role = jda.getRoleById(Constants.PLAYER);
        assert role != null;

        guild
            .addRoleToMember(member, role)
            .queue();

        player.sendMessage(Messages.INVITE_VERIFIED);
    }

    @SneakyThrows
    private User createUser(Player player, Verification verification) {
        User user = EntitySerializer.init(User.class);

        user.getUsername().init(player.getName());
        user.getUuid().init(player.getUniqueId());
        user.getDiscordId().init(verification.getDiscordId().get());

        user.getRegistered().init(System.currentTimeMillis());
        user.getLastOnline().init(System.currentTimeMillis());
        user.getPlayTime().init(0L);

        user.getInvite().set(verification.getCode().get());
        user.getGroup().init("Invited");

        return user;
    }
}
