package ee.yoursit.core.command.discord;

import dev.inventex.discord.command.slash.CommandContext;
import dev.inventex.discord.command.slash.SlashCommand;
import dev.inventex.discord.command.slash.SlashCommandInfo;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.database.entity.api.EntitySerializer;
import ee.yoursit.core.database.entity.impl.Ban;
import ee.yoursit.core.database.entity.impl.User;
import ee.yoursit.core.util.Constants;
import lombok.SneakyThrows;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.util.UUID;

@SlashCommandInfo(name = "ban", description = "Remove a player from the server.")
public class BanCommand extends SlashCommand {
    @Override
    protected void initialize() {
        addOption(
            OptionType.USER,
            "target",
            "Target user to be removed",
            true, false
        );
        addOption(
            OptionType.STRING,
            "reason",
            "The reason for the removal",
            true, false
        );
    }

    @Override
    public void execute(CommandContext context) {
        String reason = context.getString("reason");

        Guild guild = YsmpCore.INSTANCE
            .getClient()
            .getJda()
            .getGuildById(Constants.YSMP_SERVER);
        assert guild != null : "Unable to fetch YSMP guild";

        Role role = guild.getRoleById(Constants.BANNED);
        assert role != null : "Unable to fetch Banned role";

        Member member = guild.getMember(context.getUser("target"));
        assert member != null : "Unable to fetch target member";

        guild
            .addRoleToMember(member, role)
            .queue();

        String nickname = member.getNickname();
        if (nickname == null)
            return;

        Player player = Bukkit.getPlayer(nickname);
        if (player != null) {
            YsmpCore.INSTANCE
                .getDataManager()
                .get(player)
                .kick(Messages.BANNED_FROM_SERVER);
        }

        YsmpCore.INSTANCE
            .getUserService()
            .getUser(nickname)
            .then(user -> {
                Member self = context.getInteraction().getMember();
                assert self != null;

                banUser(context, user, self.getNickname(), reason);
            })
            .except(Throwable::printStackTrace);
    }

    @SneakyThrows
    private void banUser(CommandContext context, User user, String nickname, String reason) {
        user.getBanned().set(true);

        User moderator = YsmpCore.INSTANCE
            .getUserService()
            .getUser(nickname)
            .get();

        Ban ban = EntitySerializer.init(Ban.class);
        ban.getBanId().init(UUID.randomUUID());
        ban.getModerator().init(moderator.getUuid().get());
        ban.getTarget().init(user.getUuid().get());
        ban.getReason().init(reason);
        ban.getTimestamp().init(System.currentTimeMillis());

        YsmpCore.INSTANCE
            .getBanService()
            .createBan(ban);

        MessageEmbed embed = new EmbedBuilder()
            .setTitle("User Banned")
            .setColor(Constants.TRANSPARENT)
            .addField("Target", user.getUsername().get(), true)
            .addField("Moderator", moderator.getUsername().get(), true)
            .addField("Reason", reason, true)
            .build();

        context
            .getInteraction()
            .replyEmbeds(embed)
            .queue();
    }
}
