package ee.yoursit.core.command.discord;

import dev.inventex.discord.command.slash.CommandContext;
import dev.inventex.discord.command.slash.SlashCommand;
import dev.inventex.discord.command.slash.SlashCommandInfo;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.database.entity.api.EntitySerializer;
import ee.yoursit.core.database.entity.impl.Invite;
import ee.yoursit.core.database.entity.impl.User;
import ee.yoursit.core.util.Constants;
import ee.yoursit.core.util.InviteGenerator;
import lombok.SneakyThrows;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.Role;

import java.awt.*;

@SlashCommandInfo(name = "invite", description = "Create an invite code for the YSMP server")
public class InviteCommand extends SlashCommand {
    private final InviteGenerator generator = new InviteGenerator(5, 4);

    @Override
    public void execute(CommandContext context) {
        Member member = context
            .getInteraction()
            .getMember();
        assert member != null;

        YsmpCore.INSTANCE
            .getUserService()
            .getUser(member.getNickname())
            .then(user -> createInvite(context, user))
            .except(e -> handleNotUser(context));
    }

    private void createInvite(CommandContext context, User user) {
        Guild guild = context
            .getInteraction()
            .getGuild();

        Member member = context
            .getInteraction()
            .getMember();

        assert member != null;
        assert guild != null;

        int invites = user.getInvites().size();
        int maxInvites = getMaxInvites(member, guild);

        if (invites >= maxInvites) {
            exceededInvites(context);
            return;
        }

        Invite invite = newInvite(user);
        YsmpCore.INSTANCE
            .getInviteService()
            .createInvite(invite);

        MessageEmbed embed = new EmbedBuilder()
            .setTitle("Invite Created")
            .setDescription("```" + invite.getCode().get() + "```")
            .setColor(Color.GREEN)
            .build();

        context
            .getInteraction()
            .deferReply(true)
            .addEmbeds(embed)
            .queue();
    }

    private int getMaxInvites(Member member, Guild guild) {
        Role premium = guild.getRoleById(Constants.PREMIUM);
        Role business = guild.getRoleById(Constants.BUSINESS);

        assert premium != null;
        assert business != null;

        boolean hasPremiumOrHigher = member
            .getRoles()
            .stream()
            .anyMatch(role -> role.getPosition() >= business.getPosition());

        if (hasPremiumOrHigher)
            return 2;

        boolean hasBusiness = member
            .getRoles()
            .stream()
            .anyMatch(role -> role.getPosition() == premium.getPosition());

        return hasBusiness ? 1 : 0;
    }

    private void exceededInvites(CommandContext context) {
        MessageEmbed embed = new EmbedBuilder()
            .setTitle("Error")
            .setDescription(Messages.INVITE_LIMIT_EXCEEDED)
            .setColor(Color.RED)
            .build();

        context
            .getInteraction()
            .replyEmbeds(embed)
            .queue();
    }

    @SneakyThrows
    private Invite newInvite(User user) {
        Invite invite = EntitySerializer.init(Invite.class);

        invite.getCode().init(generator.generate());
        invite.getUsed().init(false);
        invite.getCreator().init(user.getUuid().get().toString());

        return invite;
    }

    private void handleNotUser(CommandContext context) {
        MessageEmbed embed = new EmbedBuilder()
            .setTitle("Error")
            .setDescription(Messages.NOT_A_USER)
            .setColor(Color.RED)
            .build();

        context
            .getInteraction()
            .deferReply(true)
            .addEmbeds(embed)
            .queue();
    }
}
