package ee.yoursit.core.command.discord;

import dev.inventex.discord.command.slash.CommandContext;
import dev.inventex.discord.command.slash.SlashCommand;
import dev.inventex.discord.command.slash.SlashCommandInfo;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.config.Messages;
import ee.yoursit.core.database.entity.api.EntitySerializer;
import ee.yoursit.core.database.entity.impl.Invite;
import ee.yoursit.core.database.entity.impl.Verification;
import ee.yoursit.core.util.Constants;
import lombok.SneakyThrows;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.interactions.commands.OptionType;

import java.awt.*;

@SlashCommandInfo(name = "verify", description = "Accept an invitation from an SMP player.")
public class VerifyCommand extends SlashCommand {
    @Override
    protected void initialize() {
        addOption(
            OptionType.STRING,
            "code",
            "The invite code generated by a user",
            true, false
        );
        addOption(OptionType.STRING,
            "username",
            "The in-game name of your Minecraft account",
            true, false
        );
    }

    @Override
    public void execute(CommandContext context) {
        Guild guild = context.getInteraction().getGuild();
        if (guild == null || guild.getIdLong() != Constants.YSMP_SERVER) {
            handleError(context, Messages.INVITE_NOT_ON_GUILD);
            return;
        }

        String code = context.getString("code");

        YsmpCore.INSTANCE
            .getInviteService()
            .getInvite(code)
            .then(invite -> acceptInvite(context, invite))
            .except(e -> handleError(context, Messages.INVITE_NOT_FOUND));
    }

    @SneakyThrows
    private synchronized void acceptInvite(CommandContext context, Invite invite) {
        if (invite.getUsed().get()) {
            handleError(context, Messages.INVITE_NOT_FOUND);
            return;
        }

        Verification existingVerification = YsmpCore.INSTANCE
            .getVerificationService()
            .getVerification(invite.getCode().get())
            .get();

        if (existingVerification != null) {
            handleError(context, Messages.INVITE_ALREADY_VERIFYING);
            return;
        }

        Member member = context
            .getInteraction()
            .getMember();
        assert member != null;

        String username = context.getString("username");
        String code = context.getString("code");

        Verification verification = EntitySerializer.init(Verification.class);
        verification.getDiscordId().init(member.getIdLong());
        verification.getUsername().init(username);
        verification.getCode().init(code);
        verification.getCreation().init(System.currentTimeMillis());

        YsmpCore.INSTANCE
            .getVerificationService()
            .addVerification(verification)
            .get();

        MessageEmbed embed = new EmbedBuilder()
            .setTitle("Verification")
            .setDescription(Messages.INVITE_VERIFICATION
                .replace("{code}", code))
            .setColor(Constants.TRANSPARENT)
            .build();

        context
            .getInteraction()
            .deferReply(true)
            .addEmbeds(embed)
            .queue();
    }

    private void handleError(CommandContext context, String message) {
        MessageEmbed embed = new EmbedBuilder()
            .setTitle("Error")
            .setDescription(message)
            .setColor(Color.RED)
            .build();

        context
            .getInteraction()
            .deferReply(true)
            .addEmbeds(embed)
            .queue();
    }
}
