package ee.yoursit.core.database.entity.api;

import org.bson.Document;
import org.bukkit.Location;

import java.lang.reflect.*;
import java.util.*;

public class EntitySerializer {
    private static final Set<Class<?>> NUMBERS = new HashSet<>();
    static {
        NUMBERS.add(byte.class);
        NUMBERS.add(short.class);
        NUMBERS.add(int.class);
        NUMBERS.add(long.class);
        NUMBERS.add(float.class);
        NUMBERS.add(double.class);
        NUMBERS.add(Byte.class);
        NUMBERS.add(Short.class);
        NUMBERS.add(Integer.class);
        NUMBERS.add(Long.class);
        NUMBERS.add(Float.class);
        NUMBERS.add(Double.class);
        NUMBERS.add(Number.class);
    }

    public static <T extends Entity> Document serialize(T entity) {
        Document document = new Document();

        for (Field field : entity.getClass().getDeclaredFields()) {
            if (Modifier.isTransient(field.getModifiers()))
                continue;

            field.setAccessible(true);

            try {
                @SuppressWarnings("rawtypes")
                EntityProperty property = (EntityProperty) field.get(entity);

                @SuppressWarnings("unchecked")
                Object serialize = serialize(property.getType(), property.get());
                document.append(field.getName(), serialize);
            } catch (Exception ignored) {
            }
        }

        return document;
    }

    public static <T> Object serialize(Class<T> type, Object object) {
        if (object == null)
            return null;

        else if (NUMBERS.contains(type))
            return type.isInstance(object)
                ? object
                : Double.parseDouble(String.valueOf(object));

        else if (object instanceof UUID)
            return object.toString();

        else if (object instanceof Location location)
            return location.serialize();

        return object;
    }

    public static <T extends Entity> T deserialize(Document document, Class<T> type) throws Exception {
        if (document == null)
            throw new IllegalStateException("Trying to deserialize null document.");

        T entity = type.getConstructor().newInstance();

        for (Field field : type.getDeclaredFields()) {
            if (Modifier.isTransient(field.getModifiers()))
                continue;

            field.setAccessible(true);

            try {
                Object value = document.get(field.getName());

                ParameterizedType genericType = (ParameterizedType) field.getGenericType();
                Class<?> propertyType = (Class<?>) genericType.getActualTypeArguments()[0];

                Object deserialize = deserialize(propertyType, value);

                @SuppressWarnings({"rawtypes", "unchecked"})
                EntityProperty property = new EntityProperty(entity, field.getName(), propertyType, deserialize);
                field.set(entity, property);
            } catch (Exception ignored) {
            }
        }

        return entity;
    }

    @SuppressWarnings("unchecked")
    public static <T> Object deserialize(Class<T> type, Object object) {
        if (object == null)
            return null;

        if (NUMBERS.contains(type))
            return type.isInstance(object)
                ? object
                : Double.parseDouble(String.valueOf(object));

        else if (UUID.class.isAssignableFrom(type))
                return UUID.fromString(String.valueOf(object));

        else if (Location.class.isAssignableFrom(type))
            return Location.deserialize((Map<String, Object>) object);
        return object;
    }

    public static <T extends Entity> T init(Class<T> type) throws Exception {
        T entity = type.getConstructor().newInstance();

        int fieldIndex = 0;
        for (Field field : type.getDeclaredFields()) {
            if (Modifier.isStatic(field.getModifiers()))
                continue;

            Class<?> fieldType = field.getType();
            if (!EntityProperty.class.isAssignableFrom(fieldType))
                continue;

            ParameterizedType genericType = (ParameterizedType) field.getGenericType();
            Class<?> propertyType = (Class<?>) genericType.getActualTypeArguments()[0];

            Constructor<?> constructor = fieldType.getConstructor(Entity.class, String.class, Class.class);

            @SuppressWarnings({"rawtypes", "unchecked"})
            Object property = constructor.newInstance(entity, field.getName(), propertyType);

            field.setAccessible(true);
            field.set(entity, property);
        }

        return entity;
    }
}
