package ee.yoursit.core.database.entity.api;

import dev.inventex.octa.concurrent.future.Future;

import java.util.ArrayList;
import java.util.List;

public class ListEntityProperty<T> extends EntityProperty<List<T>> {
    public ListEntityProperty(Entity entity, String name, Class<List<T>> type, List<T> value) {
        super(entity, name, type, value);
    }

    public ListEntityProperty(Entity entity, String name, Class<List<T>> type) {
        super(entity, name, type);
    }

    @MutateEntity
    public Future<Boolean> push(T value) {
        List<T> list = get();
        if (list == null)
            init(list = new ArrayList<>());
        boolean add = list.add(value);
        return mutate().transform(e -> add);
    }

    @MutateEntity
    public Future<Boolean> pull(T value) {
        List<T> list = get();
        if (list == null)
            return Future.completed(false);
        boolean removed = list.remove(value);
        return mutate().transform(e -> removed);
    }

    @MutateEntity
    public Future<T> pop(int index) {
        List<T> list = get();
        if (list == null)
            return Future.completed((T) null);
        T removed = list.remove(index);
        return mutate().transform(e -> removed);
    }

    public T get(int index) {
        List<T> list = get();
        if (list == null)
            return null;
        return list.get(index);
    }

    public int size() {
        List<T> list = get();
        return list != null ? list.size() : 0;
    }
}
