package ee.yoursit.core.database.service;

import com.google.common.collect.Maps;
import com.mongodb.client.FindIterable;
import com.mongodb.client.MongoCursor;
import dev.inventex.octa.concurrent.future.Future;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.database.entity.api.EntitySerializer;
import ee.yoursit.core.database.entity.impl.Group;
import lombok.Getter;
import org.bson.Document;

import java.util.Map;

public class GroupService {
    @Getter
    private final Map<String, Group> groupCache = Maps.newConcurrentMap();

    public void loadGroups() {
        try (MongoCursor<Document> cursor = YsmpCore.INSTANCE
                .getDatabase()
                .getGroups()
                .find()
                .cursor()) {
            while (cursor.hasNext()) {
                loadGroup(cursor.next());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void loadGroup(Document document) {
        try {
            Group group = EntitySerializer.deserialize(document, Group.class);
            groupCache.put(group.getName().get(), group);
        } catch (Exception e) {
            System.err.println("Unable to load group " + document);
            e.printStackTrace();
        }
    }

    public Future<Group> getGroup(String name) {
        Group group = groupCache.get(name);
        if (group != null)
            return Future.completed(group);

        return Future.tryCompleteAsync(() -> {
            Document document = YsmpCore.INSTANCE
                .getDatabase()
                .getGroups()
                .find(Group.query(name))
                .first();

            return EntitySerializer.deserialize(document, Group.class);
        });
    }

    public Future<Void> addGroup(Group group) {
        if (groupCache.containsKey(group.getName().get()))
            return Future.failed(new IllegalStateException("Group '" + group.getName() + "' already exists."));

        return Future.tryCompleteAsync(() -> {
            Document document = EntitySerializer.serialize(group);
            YsmpCore.INSTANCE
                .getDatabase()
                .getGroups()
                .insertOne(document);
        });
    }

    public Future<Void> removeGroup(String name) {
        groupCache.remove(name);

        return Future.tryCompleteAsync(() -> {
            YsmpCore.INSTANCE
                .getDatabase()
                .getGroups()
                .deleteOne(Group.query(name));
        });
    }
}
