package ee.yoursit.core.module;

import dev.inventex.discord.command.CommandManager;
import dev.inventex.discord.command.slash.SlashCommand;
import ee.yoursit.core.YsmpCore;
import ee.yoursit.core.command.discord.*;
import ee.yoursit.core.util.Constants;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.interactions.commands.Command;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

public class DiscordCommandRegistry implements CommandRegistry {
    private final Class<?>[] classes = {
        PlayersCommand.class,
        BanCommand.class,
        UnbanCommand.class,
        InviteCommand.class,
        VerifyCommand.class,
        SmpCommand.class
    };

    private final Constructor<?>[] constructors = new Constructor[classes.length];

    private final List<SlashCommand> commands = new ArrayList<>(classes.length);

    public DiscordCommandRegistry() {
        for (int i = 0; i < classes.length; i++) {
            Class<?> command = classes[i];
            try {
                constructors[i] = command.getDeclaredConstructor();
            }
            catch (NoSuchMethodException e) {
                System.err.println("Unable to load command " + command.getSimpleName() + ":");
                e.printStackTrace();
            }
        }
    }

    @Override
    public void load() {
        JDA jda = YsmpCore.INSTANCE
            .getClient()
            .getJda();

        jda
            .retrieveCommands()
            .queue(list -> {
                for (Command command : list) {
                    command
                        .delete()
                        .queue();
                }
            });

        Guild guild = jda.getGuildById(Constants.YSMP_SERVER);
        assert guild != null;

        CommandManager commandManager = YsmpCore.INSTANCE
            .getClient()
            .getCommandManager();

        for (Constructor<?> constructor : constructors) {
            try {
                SlashCommand command = (SlashCommand) constructor.newInstance();
                commands.add(command);

                if (command instanceof SmpCommand) {
                    Guild main = jda.getGuildById(Constants.MAIN_SERVER);
                    assert main != null;

                    commandManager.registerToGuild(command, main);
                    continue;
                }

                commandManager.registerToGuild(command, guild);
            }
            catch (Exception e) {
                System.err.println("Unable to instantiate check " + constructor.getDeclaringClass().getSimpleName());
                e.printStackTrace();
            }
        }
    }

    @Override
    public void unload() {
    }
}
