package ee.yoursit.core.util;

import net.md_5.bungee.api.ChatColor;
import net.md_5.bungee.api.chat.*;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Message {
    private final TextComponent component;

    public Message(String message, Object... args) {
        component = new TextComponent(translate(message, args));
    }

    public Message() {
        component = new TextComponent();
    }

    public Message click(ClickAction action, String message, Object... args) {
        ClickEvent.Action clickAction = ClickEvent.Action.valueOf(action.name());
        component.setClickEvent(new ClickEvent(clickAction, translate(message, args)));
        return this;
    }

    public Message click(String command, Object... args) {
        return click(ClickAction.RUN_COMMAND, command, args);
    }

    public Message hover(HoverAction action, String message, Object... args) {
        HoverEvent.Action hoverAction = HoverEvent.Action.valueOf(action.name());
        BaseComponent[] data = new ComponentBuilder(translate(message, args)).create();
        component.setHoverEvent(new HoverEvent(hoverAction, data));
        return this;
    }

    public Message hover(String message, Object... args) {
        return hover(HoverAction.SHOW_TEXT, message, args);
    }

    public Message add(String message, Object... args) {
        component.addExtra(translate(message, args));
        return this;
    }

    public Message add(Message message) {
        component.addExtra(message.component);
        return this;
    }

    public void send(Player player) {
        player.spigot().sendMessage(component);
    }

    public void send(CommandSender sender) {
        if (sender instanceof Player)
            ((Player) sender).spigot().sendMessage(component);
        else
            sender.sendMessage(component.toLegacyText());
    }

    public static String translate(String message, Object... args) {
        message = String.format(message, args);

        Pattern pattern = Pattern.compile("#[a-fA-F0-9]{6}");
        Matcher matcher = pattern.matcher(message);

        while (matcher.find()) {
            String color = message.substring(matcher.start(), matcher.end());
            message = message.replace(color, String.valueOf(ChatColor.of(color)));
            matcher = pattern.matcher(message);
        }

        return ChatColor.translateAlternateColorCodes('&', message);
    }
}

